##############################################################################
#
# 	Copyright (c) GeoWorks 1988 -- All Rights Reserved
#
# PROJECT:	PC GEOS
# MODULE:	Swat -- System Library
# FILE: 	thread.tcl
# AUTHOR: 	Adam de Boor, Apr 13, 1989
#
# COMMANDS:
# 	Name			Description
#	----			-----------
#   	threadstat  	    	Print info on all threads
#   	pthread	    	    	Print a thread handle
#   	freeze	    	    	Try and keep the indicated thread from running
#   	    	    	    	unless it absolutely has to.
#   	thaw	    	    	Let a thread run normally.
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	4/13/89		Initial Revision
#
# DESCRIPTION:
#	Functions for playing with things in the Thread module
#
#	$Id: thread.tcl,v 3.23 93/11/16 13:53:52 adam Exp $
#
###############################################################################

##############################################################################
#				print-thread
##############################################################################
#
# SYNOPSIS:	Callback routine for print-queue and others to print info about
#		a thread.
# PASS:		td  = value string containing the HandleThread fetched for the
#		      thread 
#   	    	id  = handle ID of the thread
# CALLED BY:	print-queue, thread-stat
# RETURN:	nothing
# SIDE EFFECTS:	none
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defsubr print-thread {td id}
{
    if {[null [handle lookup [field $td HT_owner]]]} {
    	var pname [mapconcat i
		   [value fetch
	    	    [expr [value fetch
			   kdata:[field $td HT_owner].HM_addr]*16].GH_geodeName]
		   {var i}]
    } else {
    	var pname [patient name [handle patient [handle lookup $id]]]
    }

    echo [format 
    	  {%04X owned by %04X (%-8s); base prio: %3d, usage: %3d, cur pri: %3d}
	  $id [field $td HT_owner] 
	  $pname
    	  [field $td HT_basePriority] [field $td HT_cpuUsage]
	  [field $td HT_curPriority]]
}]

##############################################################################
#				print-queue
##############################################################################
#
# SYNOPSIS:	Print out a queue of threads
# PASS:		head	= handle ID for the first thread on the queue
# CALLED BY:	threadstat
# RETURN:	nothing
# SIDE EFFECTS:	none
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defsubr print-queue {head}
{
    [for {var cur $head} {$cur} {var cur [field $td HT_nextQThread]}
    {
    	var td [value fetch kdata:$cur HandleThread]
    	print-thread $td $cur
    }]
}]

##############################################################################
#				threadstat
##############################################################################
#
# SYNOPSIS:	Print out info on the threads active in the system
# PASS:		nothing
# CALLED BY:	user
# RETURN:	nothing
# SIDE EFFECTS:	none
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defcmd threadstat {} {system.thread thread}
{Usage:
    threadstat

Examples:
    "threadstat"

Synopsis:
    Provides information about all threads and various thread queues and
    synchronization points in the system.

Notes:

See also:
    ps
}
{
    #
    # Fetch the current thread's id (in decimal)
    #
    var	thread [read-reg curThread]
    #
    # Just for kicks, find the patient it belongs to and its number w.r.t. that
    # patient.
    #
    var handle [handle lookup $thread]

    echo [format {Current thread: %04xh (%s #%d)} $thread
    	    [patient name [handle patient $handle]]
	    [thread number [handle other $handle]]]

    #
    # This isn't strictly necessary, since Swat knows about all threads in
    # the system ([thread all] will return tokens for them all).
    #
    echo {List of threads:}
    [for {var cur [value fetch threadListPtr]}
    	 {$cur}
    	 {var cur [field $td HT_next]}
    {
    	var td [value fetch kdata:$cur HandleThread]
    	print-thread $td $cur
    }]
    
    echo
    echo {Run queue:}

    print-queue [value fetch runQueue]

    echo
    [map {label lock}
    	[if {[not-1x-branch]}
	 {list Heap DOS/BIOS Geode}
    	 {list Heap Directory {Working Directory} {File List} DOS/BIOS Geode {Disk Table}}]
	[if {[not-1x-branch]}
	 {list heapSem biosLock geodeSem}
	 {list heapSem dirLock cwdLock sftLock dosLock geodeSem diskLock}]
    {
	pmodulelock [concat $label lock] [value fetch $lock]
    }]
}]

##############################################################################
#				pmodulelock
##############################################################################
#
# SYNOPSIS:	Print who owns a module lock and any threads queued on it
# PASS:		label	= label for the lock
#   	    	ml  = value list for the ThreadLock structure
# CALLED BY:	threadstat, user
# RETURN:	nothing
# SIDE EFFECTS:	none
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defsubr pmodulelock {label ml}
{
    global geos-release
    if {${geos-release} >= 2} {
	if {[null [field $ml TL_owner]]} {
	    echo [format {%30s: value = %d} $label 
		    [field [field $ml TL_sem] Sem_value]]
	} elif {[field $ml TL_owner] == 65535} {
	    echo [format {%30s: FREE} $label]
	} else {
	    echo [format {%30s: owned by: %s, value = %d} $label
		    [patient name [handle patient [handle lookup
						    [field $ml TL_owner]]]]
		    [field [field $ml TL_sem] Sem_value]]
	}
	#
	# Print any threads blocked on the queue
	#
	var q [field [field $ml TL_sem] Sem_queue]
	if {$q > 15} {
	    print-queue $q
	    echo
	}
    } else {
	if {[null [field $ml ML_owner]]} {
	    echo [format {%30s: value = %d} $label 
		    [field [field $ml ML_sem] Sem_value]]
	} elif {[field $ml ML_owner] == 65535} {
	    echo [format {%30s: FREE} $label]
	} else {
	    echo [format {%30s: owned by: %s, value = %d} $label
		    [patient name [handle patient [handle lookup
						    [field $ml ML_owner]]]]
		    [field [field $ml ML_sem] Sem_value]]
	}
	#
	# Print any threads blocked on the queue
	#
	var q [field [field $ml ML_sem] Sem_queue]
	if {$q > 15} {
	    print-queue $q
	    echo
	}
    }
}]

##############################################################################
#				pthread
##############################################################################
#
# SYNOPSIS:	Print information about a thread
# PASS:		id  = handle ID for the thread
# CALLED BY:	user, phandle
# RETURN:	nothing
# SIDE EFFECTS:	none
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defcmd pthread {id} {system.thread thread}
{Usage:
    pthread <id>

Examples:
    "pthread 16c0h" 	Prints information about the thread whose handle is
    	    	    	16c0h

Synopsis:
    Provides various useful pieces of information about a particular thread,
    including its current priority and its current registers.

Notes:
    * <id> is the thread's handle ID, as obtained with the "ps -t" or
      "threadstat" command.

See also:
    ps, threadstat
}
{
    global geos-release

    var id [getvalue $id]
    var handle [handle lookup $id]
    if {[null $handle] || ![handle isthread $handle]} {
    	error [format {%s not a valid thread ID} $id]
    }

    var td [value fetch kdata:$id HandleThread]

    echo [format {Owner: %04xh (%s)}
    	    [field $td HT_owner] [patient name [handle patient $handle]]]
    var nhandle [handle lookup [field $td HT_nextQThread]]
    echo [format {nextQueued: %s, next = %04xh}
    	    [if {[field $td HT_nextQThread]} {
    	    	[format {%04xh (%s #%d)}
    	    	    [field $td HT_nextQThread]
    	    	    [patient name [handle patient $nhandle]]
    	    	    [thread number [handle other $nhandle]]]
    	     } else {
    	    	[format nil]
    	    }]
    	    [field $td HT_next]]
    echo [format {base priority: %3d, cpu usage: %3d, current priority: %3d}
    	    [field $td HT_basePriority]
    	    [field $td HT_cpuUsage]
	    [field $td HT_curPriority]]

    if {[field $td HT_saveSS]} {
        echo Registers:
        var thread [handle other $handle]
        var j 0
        foreach i {CX DX SP BP SI DI} {
        	var regval [thread register $thread $i]
        	echo -n [format {%-4s%04xh%8d} $i $regval $regval]
    	var j [expr ($j+1)%3]
    	if {$j == 0} {echo} else {echo -n \t}
        }
        foreach i {CS DS SS ES} {
        	var regval [thread register $thread $i]
        	var handle [handle find [format %04xh:0 $regval]]
        	if {![null $handle]} {
        	    if {[handle state $handle] & 0x80} {
        	    	#
    			# Handle is a resource handle, so it's got a symbol in
			# its otherInfo field. We want its name.
        	    	#
        	    	echo [format {%-4s%04xh   handle %04xh (%s)}
        	    	    	    $i $regval [handle id $handle]
        	    	    	    [symbol fullname [handle other $handle]]]
        	    } else {
        	    	echo [format {%-4s%04xh   handle %04xh}
        	    	    	    $i $regval [handle id $handle]]
        	    }
        	} else {
        	    echo [format {%-4s%04xh   no handle} $i $regval]
        	}
        }
    } else {
    	echo No registers saved
    }
}]


##############################################################################
#				freeze
##############################################################################
#
# SYNOPSIS:	Keep a thread from running unless it's the only thing in the
#   	    	system that's runnable.
# PASS:		who = an argument similar to that passed to "switch"
# CALLED BY:	user
# RETURN:	nothing
# SIDE EFFECTS:	the thread's curPriority and basePriority are set to 255 until
#   	    	"thaw" is executed.
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defcommand freeze {{who {}}} {thread patient.running}
{Usage:
    freeze [ <patient> | :<n> | <patient>:<n> | <id> ]

Examples:
    "freeze term"   	Freezes the application thread for "term"
    "freeze :1"	    	Freezes thread #1 of the current patient
    "freeze 16c0h"  	Freezes the thread whose handle is 16c0h
    "freeze"		Freezes the current thread

Synopsis:
    Freezing a thread prevents a thread from running unless it's the only thread
    that's runnable in the entire system.

Notes:
    * A frozen thread is not dead in the water, as it will still run if nothing
      else is runnable.

    * Freezing a thread is most useful when debugging multi-threaded applica-
      tions where a bug appears to be caused by a timing problem or race
      condition between the two threads. Freezing one of the threads ensures
      a consistent timing relationship between the two threads and allows the
      bug to be reproduced much more easily.

    * The freezing of a thread is accomplished by setting its base and current
      priorities to as high a number as possible (255) thereby making the
      thread the least-favored thread in the system. The previous priority
      can be restored using the "thaw" command.

    * If you give no argument, Swat will freeze the current thread, which
      may or may not be the thread the target system thinks is the
      current thread.

See also:
    thaw
}
{
    var curp [patient data]

    if {![null $who]} {
    	switch $who
    }

    var desired [patient data]

    foreach i [patient threads] {
	if {[thread number $i] == [index $desired 2]} {
	    break
	}
    }
    
    #
    # Save old priority
    #
    global _old_$desired
    var id [thread id $i]
    var bp [value fetch kdata:$id.geos::HT_basePriority]
    if {$bp == 255} {
    	error [format {%s is already frozen} $who]
    }
    var _old_$desired $bp
    #
    # Increase both the base and the current priority to maximum.
    #
    assign kdata:$id.geos::HT_curPriority 255
    assign kdata:$id.geos::HT_basePriority 255
    #
    # Switch back to previous "current patient"
    #
    switch [index $curp 0]:[index $curp 2]
}]

##############################################################################
#				thaw
##############################################################################
#
# SYNOPSIS:	Allow a "frozen" thread to run normally
# PASS:		who = an argument similar to that passed to "switch"
# CALLED BY:	user
# RETURN:	nothing
# SIDE EFFECTS:	the thread's base priority is returned to its previous value
#   	    	and its current priority recalculated properly
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defcommand thaw {{who {}}} {thread patient.running}
{Usage:
    thaw [ <patient> | :<n> | <patient>:<n> | <id> ]

Examples:
    "thaw term"   	Allows the application thread for "term" to run normally
    "thaw :1"	    	Allows thread #1 of the current patient to run normally
    "thaw 16c0h"  	Allows the thread whose handle is 16c0h to run normally
    "thaw"		Allows the current thread to run normally

Synopsis:
    Thawing a thread restores its priority to what it was before the thread
    was frozen.

Notes:
    * If you give no argument, Swat will thaw the current thread, which
      may or may not be the thread the target system thinks is the
      current thread.

See also:
    freeze
}
{
    var curp [patient data]

    if {![null $who]} {
    	switch $who
    }

    var desired [patient data]

    foreach i [patient threads] {
	if {[thread number $i] == [index $desired 2]} {
	    break
	}
    }
    
    var id [thread id $i]
    #
    # Make sure the thread is actually frozen
    #
    global _old_$desired
    var bp [value fetch kdata:$id.geos::HT_basePriority]
    if {$bp != 255} {
    	error [format {%s isn't frozen} $who]
    }
    #
    # Reset its base priority to what it was before
    #
    assign kdata:$id.geos::HT_basePriority [var _old_$desired]

    #
    # Use that and its recent CPU usage to calculate the proper current
    # priority for the thread, handling wrap-around correctly
    #
    var cp [expr [value fetch kdata:$id.geos::HT_cpuUsage]+[var _old_$desired]]
    if {$cp > 255} {var cp 255}

    assign kdata:$id.geos::HT_curPriority $cp

    #
    # Switch back to previous "current patient"
    #
    switch [index $curp 0]:[index $curp 2]
}]

##############################################################################
#				antifreeze
##############################################################################
#
# SYNOPSIS:	Make a thread the most-likely thing to run.
# PASS:		who = an argument similar to that passed to "switch"
# CALLED BY:	user
# RETURN:	nothing
# SIDE EFFECTS:	the thread's curPriority and basePriority are set to 0 until
#   	    	"thaw" is executed.
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	12/12/92	Initial Revision
#
##############################################################################
[defcommand antifreeze {{who {}}} thread
{Usage:
    antifreeze <patient>
    antifreeze :<n>
    antifreeze <patient>:<n>
    antifreeze <id>

Examples:
    "antifreeze term"   	Promotes the application thread for "term"
				to be the "most-runnable"
    "antifreeze :1"	    	Does likewise for thread #1 of the current
				patient
    "antifreeze 16c0h"  	Does likewise the thread whose handle is 16c0h
    "antifreeze"    	    	Promotes the current thread to be the
				"most-runnable."

Synopsis:
    Antifreezing a thread makes a thread the most-favored thread in the entire
    system. Should it ever be runnable, it will be the thread the system
    runs, possibly to the detriment of other threads...

Notes:
    * The antifreezing of a thread is accomplished by setting its base and
      current priorities to as low a number as possible (0) thereby making the
      thread the most-favored thread in the system. The previous priority
      can be restored using the "antithaw" command.

See also:
    antithaw
}
{
    var curp [patient data]

    if {![null $who]} {
    	switch $who
    }

    var desired [patient data]

    foreach i [patient threads] {
	if {[thread number $i] == [index $desired 2]} {
	    break
	}
    }
    
    #
    # Save old priority
    #
    global _old_$desired
    var id [thread id $i]
    var bp [value fetch kdata:$id.geos::HT_basePriority]
    if {$bp == 0} {
    	error [format {%s is already anti-frozen} $who]
    }
    var _old_$desired $bp
    #
    # Decrease both the base and the current priority to minimum.
    #
    assign kdata:$id.geos::HT_curPriority 0
    assign kdata:$id.geos::HT_basePriority 0
    #
    # Switch back to previous "current patient"
    #
    switch [index $curp 0]:[index $curp 2]
}]

##############################################################################
#				antithaw
##############################################################################
#
# SYNOPSIS:	Allow an "antifrozen" thread to run normally
# PASS:		who = an argument similar to that passed to "switch"
# CALLED BY:	user
# RETURN:	nothing
# SIDE EFFECTS:	the thread's base priority is returned to its previous value
#   	    	and its current priority recalculated properly
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	2/27/90		Initial Revision
#
##############################################################################
[defcommand antithaw {{who {}}} thread
{Usage:
    antithaw <patient>
    antithaw :<n>
    antithaw <patient>:<n>
    antithaw <id>

Examples:
    "antithaw term"   	Allows the application thread for "term" to run normally
    "antithaw :1"	Allows thread #1 of the current patient to run normally
    "antithaw 16c0h"  	Allows the thread whose handle is 16c0h to run normally

Synopsis:
    Thawing a thread restores its priority to what it was before the thread
    was given antifreeze.

Notes:

See also:
    antifreeze
}
{
    var curp [patient data]

    if {![null $who]} {
    	switch $who
    }

    var desired [patient data]

    foreach i [patient threads] {
	if {[thread number $i] == [index $desired 2]} {
	    break
	}
    }
    
    var id [thread id $i]
    #
    # Make sure the thread is actually anti-frozen
    #
    global _old_$desired
    var bp [value fetch kdata:$id.geos::HT_basePriority]
    if {$bp != 0} {
    	error [format {%s isn't anti-frozen} $who]
    }
    #
    # Reset its base priority to what it was before
    #
    assign kdata:$id.geos::HT_basePriority [var _old_$desired]

    #
    # Use that and its recent CPU usage to calculate the proper current
    # priority for the thread, handling wrap-around correctly
    #
    var cp [expr [value fetch kdata:$id.geos::HT_cpuUsage]+[var _old_$desired]]
    if {$cp > 255} {var cp 255}

    assign kdata:$id.geos::HT_curPriority $cp

    #
    # Switch back to previous "current patient"
    #
    switch [index $curp 0]:[index $curp 2]
}]

