COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:	PC GEOS
MODULE:		KBD -- Keyboard handling routines for Com43
FILE:		kbd.asm

AUTHOR:		Adam de Boor

ROUTINES:
	Name		Description
	----		-----------
	KbdInit		Initialize the module
	KbdExit		Deinitialize the module
	KbdRead		Read a character from the keyboard, if any
	KbdKeypadStart	Transmit special escape codes for keypad
	KbdKeypadEnd	Go back to numbers from keypad
	KbdGetMods	Return the current state of the modifiers.

	
REVISION HISTORY:
	Name	Date	Description
	----	----	-----------
	adam	6/8/88	Initial Revision


DESCRIPTION:
	The routines in this module manage the IBM PC keyboard. 

	The Ctrl, Alt and Shift modifiers are supported on all keys.
	The SysReq key is taken as another modifier for delivering commands
	to this program. Function keys will eventually be mappable, but for
	now they produce only those strings listed in the termcap.

	$Id: kbd.asm,v 2.5 92/11/10 17:36:01 jimmy Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
KBDMODULE	=	1
		include	com43.def
;KBD_DEBUG	= 1
program		segment
;
; Keyboard controller ports
;
KBDDATAPORT	equ	60h	; input received here and data sent from here
KBDSTATPORT	equ	64h	; keyboard status, on read:
				; bit 4 -- if 0, keyboard is inhibited
IN_BUF_FULL	equ	02h	; bit 1 -- if 1, input buffer (input to
				;	keyboard, that is) is full
OUT_BUF_FULL	equ	01h	; bit 0 -- if 1, output buffer (i.e. input
				;	from keyboard) is full.
KBDCMDPORT	equ	64h	; port to send commands to the controller (as
				; opposed to the keyboard).

; Controller commands
KBDGETCMD	equ	20h	; Get controller command byte. Format is
				; described below.
KBDSETCMD	equ	60h	; Set controller command byte:
				; bit 6 -- PC compatibility mode (?)
				; bit 5 -- PC mode (?)
				; bit 4 -- Disable keyboard
				; bit 3 -- disable keyboard inhibit
				; bit 2 -- system flag (?)
				; bit 1 -- MBZ
				; bit 0 -- enable data-ready interrupt
KBDSETLEDCMD	equ	0edh	; Set the state of the leds:
				; bit 2 -- caps lock
				; bit 1 -- num lock
				; bit 0 -- scroll lock
KBDRESETCMD	equ	0feh	; Actually the 'strobe output port bit 0'
				; command, but the processor's reset line is
				; attached to bit 0, so this command resets
				; the processor
KBDDISABLECMD	equ	0adh	; Disable the keyboard
KBDENABLECMD	equ	0aeh	; Enable the keyboard

; Keyboard commands
KBDSETDEFAULT	equ	0f6h	; set kbd to power-on defaults
KBDENABLEKBDCMD	equ	0f4h	; enable keyboard scanning
KBDRESENDCMD	equ	0feh	; resend last transmission
KBDACKCMD	equ	0fah	; command acknowledged
KBDBREAKCMD	equ	0f0h	; key release coming up
KBDOVERRUNCMD	equ	00h	; overran keyboard's buffer

lastByteSent	db	0	; Previous output byte in case of RESEND
				; request.
; Keyboard key-state flags
kbdKeyState	db	0
		bugsym	kbdKeyState
KBDLEFTKEY	equ	01h	; Left shift key down
KBDRIGHTKEY	equ	02h	; Right shift key down
KBDMETAKEY	equ	04h	; Meta (ALT) key down
KBDSYSKEY	equ	08h	; SysReq key down
KBDCTRLKEY	equ	10h	; Control key

; State bits
kbdFlags	db	0
		bugsym	kbdFlags
KBDSHIFT	equ	01h	; Shift to second keycap (if any on key)
KBDMETA		equ	02h	; Set high bit
KBDCTRL		equ	04h	; Clear bits 6 and 5
KBDSYS		equ	08h	; Special handling
KBDCAPS		equ	10h	; Extra shift for letters if caps-lock on
KBDSETTINGLED	equ	40h	; Setting LEDs. Should send LED byte if receive
				; an ACK from the keyboard.
KBDKEYPADON	equ	80h	; Keypad should produce escape codes

KBDMODFLAGS	equ	0fh	; Mask of modifier flags in kbdFlags

;
; Translation table to use.
;
kbdTransTable	dw	kbdKeys	; Translation table to use.

;
; kbdStateMap is a mapping from the modifier key bits kept in kbdKeyState to
; the state flags kept in kbdFlags. The entries are the bits that should be
; on in kbdFlags for the given modifier bit set.
;
kbdStateMap	db	0
		; KBDLEFTKEY, KBDRIGHTKEY, KBDLEFTKEY and KBDRIGHTKEY
		db	KBDSHIFT
		db	KBDSHIFT
		db	KBDSHIFT
		; KBDMETAKEY and above
		db	KBDMETA
		db	KBDMETA OR KBDSHIFT
		db	KBDMETA OR KBDSHIFT
		db	KBDMETA OR KBDSHIFT
		; KBDSYSKEY and above
		db	KBDSYS
		db	KBDSYS OR KBDSHIFT
		db	KBDSYS OR KBDSHIFT
		db	KBDSYS OR KBDSHIFT
		db	KBDSYS OR KBDMETA
		db	KBDSYS OR KBDMETA OR KBDSHIFT
		db	KBDSYS OR KBDMETA OR KBDSHIFT
		db	KBDSYS OR KBDMETA OR KBDSHIFT
		; KBDCTRLKEY and above
		db	KBDCTRL
		db	KBDCTRL OR KBDSHIFT
		db	KBDCTRL OR KBDSHIFT
		db	KBDCTRL OR KBDSHIFT
		db	KBDCTRL OR KBDMETA
		db	KBDCTRL OR KBDMETA OR KBDSHIFT
		db	KBDCTRL OR KBDMETA OR KBDSHIFT
		db	KBDCTRL OR KBDMETA OR KBDSHIFT
		db	KBDCTRL OR KBDSYS
		db	KBDCTRL OR KBDSYS OR KBDSHIFT
		db	KBDCTRL OR KBDSYS OR KBDSHIFT
		db	KBDCTRL OR KBDSYS OR KBDSHIFT
		db	KBDCTRL OR KBDSYS OR KBDMETA
		db	KBDCTRL OR KBDSYS OR KBDMETA OR KBDSHIFT
		db	KBDCTRL OR KBDSYS OR KBDMETA OR KBDSHIFT
		db	KBDCTRL OR KBDSYS OR KBDMETA OR KBDSHIFT

;
; Saved keyboard state
;
kbdVector	dw	?, ?	; Initial interrupt vector
KBDINTVEC	equ	9	; Vector number of keyboard
kbdCmdByte	db	0	; Initial keyboard state

;
; Input buffer
;
KBBUFSIZE		=	64	; Sixty-four bytes for now
KBBUFFER		STRUC
kb_head		dw	0
kb_tail		dw	0
kb_data		db	KBBUFSIZE DUP(?)
KBBUFFER		ENDS

kbdBuf		KBBUFFER	<>
		bugsym	kbdBuf

;
; Keyboard definition. The table is indexed by system scan code, as returned
; by reading KBDDATAPORT. The entries are the ascii code for the key, except
; in the case of function keys, keypad keys and modifiers (shift, alt, etc.)
; These special keys are denoted by having their high bit set. For the keypad
; keys, the low 7 bits contain the appropriate ascii codes. For the modifiers,
; the low 7 bits are the constants defined below.
;
KP0		= '0'
KP1		= '1' OR 80h
KP2		= '2' OR 80h
KP3 		= '3' OR 80h
KP4		= '4' OR 80h
KP5		= '5'
KP6		= '6' OR 80h
KP7		= '7' OR 80h
KP8		= '8' OR 80h
KP9		= '9' OR 80h
KPM		= '-'
KPP		= '+'
KPD		= '.' OR 80h
F1		= 0f1h
F2		= 0f2h
F3		= 0f3h
F4		= 0f4h
F5		= 0f5h
F6		= 0f6h
F7		= 0f7h
F8		= 0f8h
F9		= 0f9h
F10		= 0fah
F11		= 0fbh
F12		= 0fch
MODLOW		= 0e0h		; Bottom of modifier range
MODHIGH		= 0e5h		; Top of modifier range
LSH		= 0e0h		; Left SHIFT key
RSH		= 0e1h		; Right SHIFT key
ALT		= 0e2h		; ALT key
CTL		= 0e3h		; CTRL key
SYS		= 0e4h		; SysReq key
SCL		= 0e5h		; Scroll Lock key
CPL		= 0e6h		; Caps Lock key
NML		= 0e7h		; Num Lock key
kbdKeys		db	0
		db	1bh, '1', '2', '3', '4', '5', '6', '7', '8' ; 01 - 09
		db	'9', '0', '-', '=', 08h, 09h, 'q', 'w', 'e' ; 0a - 12
		db	'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']' ; 13 - 1b
		db	0dh, CTL, 'a', 's', 'd', 'f', 'g', 'h', 'j' ; 1c - 24
		db	'k', 'l', ';', "'", '`', LSH, '\\','z', 'x' ; 25 - 2d
		db	'c', 'v', 'b', 'n', 'm', ',', '.', '/', RSH ; 2e - 36
		db	  0, ALT, ' ', CPL,  F1,  F2,  F3,  F4,  F5 ; 37 - 3f
		db	 F6,  F7,  F8,  F9, F10, NML, SCL, KP7, KP8 ; 40 - 48
		db	KP9, KPM, KP4, KP5, KP6, KPP, KP1, KP2, KP3 ; 49 - 51
		db	KP0, KPD, SYS,   0,   0, F11, F12	    ; 52 - 58
KBDNUMKEYS	equ	$ - kbdKeys

altKbdKeys	db	0
		db	1bh, '1', '2', '3', '4', '5', '6', '7', '8' ; 01 - 09
		db	'9', '0', '-', '=', 08h, 09h, 'q', 'w', 'e' ; 0a - 12
		db	'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']' ; 13 - 1b
		db	0dh, CPL, 'a', 's', 'd', 'f', 'g', 'h', 'j' ; 1c - 24
		db	'k', 'l', ';', "'", '`', LSH, '\\','z', 'x' ; 25 - 2d
		db	'c', 'v', 'b', 'n', 'm', ',', '.', '/', RSH ; 2e - 36
		db	  0, ALT, ' ', CTL,  F1,  F2,  F3,  F4,  F5 ; 37 - 3f
		db	 F6,  F7,  F8,  F9, F10, NML, SCL, KP7, KP8 ; 40 - 48
		db	KP9, KPM, KP4, KP5, KP6, KPP, KP1, KP2, KP3 ; 49 - 51
		db	KP0, KPD, SYS,   0,   0, F11, F12	    ; 52 - 58

;
; Mapping of modifier key codes (LSH...SCL) to KeyState bits
;
kbdModifiers	db	KBDLEFTKEY
		db	KBDRIGHTKEY
		db	KBDMETAKEY
		db	KBDCTRLKEY
		db	KBDSYSKEY
		db	KBDSYSKEY

;
; kbdShiftTable contains the ascii code to use if the shift key is down.
; Most of the punctuation marks and all the entries for the upper-case letters
; will never be used, but they are there so I don't get too confused.
; Table is, naturally, indexed by ascii code.
;
kbdShiftTable	db	00h, 01h, 02h, 03h, 04h, 05h, 06h, 07h
		db	7fh, 09h, 0ah, 0bh, 0ch, 0dh, 0eh, 0fh
		db	10h, 11h, 12h, 13h, 14h, 15h, 16h, 17h
		db	18h, 19h, 1ah, 1bh, 1ch, 1dh, 1eh, 1fh
		db	' ', '!', '"', '#', '$', '%', '&', '"'
		db	'(', ')', '*', '+', '<', '_', '>', '?'
		db	')', '!', '@', '#', '$', '%', '^', '&'
		db	'*', '(', ':', ':', '<', '+', '>', '?'
		db	'@', 'A', 'B', 'C', 'D', 'E', 'F', 'G'
		db	'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O'
		db	'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W'
		db	'X', 'Y', 'Z', '{', '|', '}', '^', '_'
		db	'~', 'A', 'B', 'C', 'D', 'E', 'F', 'G'
		db	'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O'
		db	'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W'
		db	'X', 'Y', 'Z', '{', '|', '}', '~', 7fh
;
; Function key translations.
; Each function key has a table of 8 pointers -- one for each permutation
; of Ctrl, Alt and Shift. If the entry is 0, there is no user-defined string
; for that combination, so we use the standard one. Otherwise we insert the
; given string into the input stream.
;
kbdFuncTrans	dw	8 DUP(0)		; Strings for F1
		dw	8 DUP(0)		; F2
		dw	8 DUP(0)		; F3
		dw	8 DUP(0)		; F4
		dw	8 DUP(0)		; F5
		dw	8 DUP(0)		; F6
		dw	8 DUP(0)		; F7
		dw	8 DUP(0)		; F8
		dw	8 DUP(0)		; F9
		dw	8 DUP(0)		; F10
		dw	8 DUP(0)		; F11
		dw	8 DUP(0)		; F12
;
; Keypad translations
;
kbdKeypadTrans	label	word
		db	'k','0'			; KP0 -- nothing special
		db	'e','n'			; KP1 -- End
		dw	'd'			; KP2 -- down arrow
		db	'p','d'			; KP3 -- page down
		dw	'l'			; KP4 -- left arrow
		db	'k','5'			; KP5 -- nothing special
		dw	'r'			; KP6 -- right arrow
		dw	'h'			; KP7 -- home
		dw	'u'			; KP8 -- up arrow
		db	'p','u'			; KP9 -- page up

KbdPrintByte	proc 	near
ifdef KBD_DEBUG
		call	ScrPrintByte
endif
		ret
KbdPrintByte	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdInit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize the module

CALLED BY:	Main

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Install our interrupt vector and initialize our state.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	KbdInit
cmdbytemsg	db	"Initial command byte: ", 0
initoutbufvalmsg db	"Initial output buffer value: ", 0
kbdFlag		db	'k', 0
KbdInit		proc	near
		mov	kbdFlags, KBDSETTINGLED
		mov	kbdKeyState, 0
		
		test	sysFlags, MASK SF_PC	; Check for on base PC.
		jz	KInit1			; if not then do AT init.
		;
	        ; Running on a PC -- use the alternate keyboard encoding unless
		; given the /k flag...
		;
		mov	di, offset kbdFlag
		clr	bx
		call	FetchArg
		tst	bx
		jnz	Kjmpfarther		; /k given -- leave alone.
		mov	kbdTransTable, offset altKbdKeys
Kjmpfarther:
		jmp	KSetVectors	; skip AT init stuff

KInit1:
		;
		; Running on an AT -- use the alternate keyboard encoding if
		; given the /k flag...
		;
		mov	di, offset kbdFlag
		clr	bx
		call	FetchArg
		tst	bx
		jz	KInit1_5		; /k not given -- leave alone
		mov	kbdTransTable, offset altKbdKeys
KInit1_5:
		cli				; no interrupts while doing this

		clr	cx			; Initialize timeout value
						;  for all subsequent
						;  KReadData calls.
		;
		; Fetch initial keyboard state after clearing out the output
		; buffer.
		;
		call	KReadData		; read data port
		mov	bl, al			; save initial state byte
ifdef KBD_DEBUG
		push	bx
		push	ax
		lea	bx, initoutbufvalmsg
		call	ScrWriteString
		pop	ax
		call	ScrPrintByte
		mov	al, cr
		call	ScrWrite
		mov	al, 0ah
		call	ScrWrite
		pop	bx
endif 

		; send a "GETCMD" command to the controller

		mov	al, KBDGETCMD
		call	KSendCmd

		call	KReadData		; read result of GETCMD
ifdef KBD_DEBUG
		push	ax
		lea	bx, cmdbytemsg
		call	ScrWriteString
		pop	ax

		push	ax
		call	KbdPrintByte
		mov	al, cr
		call	ScrWrite
		mov	al, 0ah
		call	ScrWrite
		pop	ax
endif
		; kill interrupts while setting kbd initial state

		dsi
		mov	kbdCmdByte, al		; save initial command byte

		mov	al, KBDSETCMD
		call	KSendCmd		; send "SETCMD" to controller

		mov	al, 45h			; write bits for SETCMD
		call	KSendData

KSetVectors:
		;
		; Finally install our interrupt handler. Also re-enables
		; interrupts.
		;
		mov	ax, KBDINTVEC
		mov	bx, offset kbdVector
		mov	dx, offset KbdInterrupt
		call	SetInterrupt
		
		;
		; Make sure keyboard interrupt enabled in 8259
		;

		in	al, ICMASKPORT
		and	al, 11111101b	; Keyboard interrupts at level 1
		out	ICMASKPORT, al
		mov	al, 61h		; Send specific end-of-interrupt
		out	ICEOIPORT, al	; for this level.
		;
		; Set LEDs initially
		;
		mov	al, KBDSETLEDCMD
		mov	lastByteSent, al
		call	KSendData

		ret
KbdInit		endp


;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		; utility routines to send byte to data/command ports
		; al = data
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
KSendData	proc	near
		push	cx			; save a reg
		mov	ah, al			; save byte to send
		pushf				; save interrupt status
		cli				; disable ints all this time
		clr	cx			; set some timeout value
KSDwait:
		in	al, KBDSTATPORT
		test	al, IN_BUF_FULL		; wait till buffer emptied
		loopnz	KSDwait

		mov	al, ah			; restore byte to send
		out	KBDDATAPORT, al		;   and send it
		popf				; restore ints
		pop	cx			; save a reg
		ret
KSendData	endp

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
KSendCmd	proc	near
		push	cx			; save a reg
		mov	ah, al			; save byte to send
		pushf				; save interrupt status
		cli				; disable ints all this time
		clr	cx			; set some timeout value
KSCwait:
		in	al, KBDSTATPORT
		test	al, IN_BUF_FULL		; wait till buffer emptied
		loopnz	KSCwait

		mov	al, ah			; restore byte to send
		out	KBDCMDPORT, al		;   and send it
		popf				; restore ints
		pop	cx			; save a reg
		ret
KSendCmd	endp
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

KReadData	proc	near
		push	cx			; save timeout count
KRDwait:
		in	al, KBDSTATPORT		; check status
		test	al, OUT_BUF_FULL	; check output buffer bit
		loopz	KRDwait
		in	al, KBDDATAPORT		; ready, read the data
		pop	cx			; restore timeout counter
		ret
KReadData	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdExit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Clean up after ourselves

CALLED BY:	Exit

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Does nothing for now

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	KbdExit
KbdExit		proc	near
		mov	ax, KBDINTVEC
		mov	bx, offset kbdVector
		call	ResetInterrupt

		mov	al, KBDSETCMD
		call	KSendCmd
		mov	al, kbdCmdByte
		call	KSendData
		ret
KbdExit		endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdStuffChar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Place a character in the ring buffer

CALLED BY:	KbdInterrupt, KbdHandleFunc

PASS:		AL	= character to stuff

RETURN:		Nothing

DESTROYED:	BX

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/13/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

KbdStuffChar	proc	near
		mov	bx, kbdBuf.kb_tail
		mov	kbdBuf.kb_data[bx], al
		inc	bx
		cmp	bx, KBBUFSIZE
		jne	KbdStuffReturn
		clr	bx
KbdStuffReturn:
		mov	kbdBuf.kb_tail, bx
		ret
KbdStuffChar	endp
		bugsym	KbdStuffChar



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdInterrupt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle a keyboard interrupt

CALLED BY:	SPONTANEOUS

PASS:		CS	= this code segment

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Fetch the scan code from the keyboard.
	Translate it via the kbdKeys table.
	If it's special (bit 7 set), go handle it specially.
	Else modify it according to our current state and stuff the character
		in the buffer.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/13/88		Initial Revision
	adam	7/21/88		Changed to handle the setting of L(*s

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ifdef KBD_DEBUGE
resendingmsg db	"resending: ", 0
endif
KbdInterrupt	proc	near
		push	ds
		push	ax
		push	bx

		eni		; Allow other interrupts
		
		mov	ax, cs
		mov	ds, ax

		call	ScrEnable

		in	al, KBDDATAPORT		; Fetch scan code
		push	ax
		test	sysFlags, MASK SF_PC
		jz	KI1_5
		;
		; On a PC so we need to acknowledge/reset the latches by
		; strobing the high bit of port 61h
		;
		in	al, KBD_CONTROL
		mov	ah, al
		or	al, 80h
		out	KBD_CONTROL, al
		mov	al, ah
		out	KBD_CONTROL, al
		pop	ax
		push	ax
KI1_5:
		call	KbdPrintByte
		pop	ax
		
		cmp	al, KBDACKCMD
		jne	KI2
		jmp	KIGotAck
KI2:
		cmp	al, KBDRESENDCMD
		jne	KI3
ifdef KBD_DEBUGE
		mov	bx, offset resendingmsg
		call	ScrWriteString
		mov	al, lastByteSent
		call	KbdPrintByte
		mov	al, cr
		call	ScrWrite
		mov	al, 0ah
		call	ScrWrite
endif		
		mov	al, lastByteSent
		out	KBDDATAPORT, al
		jmp	KbdDiscard
KI3:
		mov	ah, al
		and	al, 07fh
		cmp	al, KBDNUMKEYS
		jae	KbdDiscard
		mov	bx, kbdTransTable
		xlatb
		tst	ah
		js	KbdIntRelease
		tst	al
		jz	KbdDiscard		; => bogus keystroke
		js	KbdIntSpecial		; bit 7 set => special handling

		;
		; Modify according to current modifier keys
		;
		test	kbdFlags, KBDSHIFT
		jz	KbdTestCtl
		lea	bx, kbdShiftTable
		xlatb
KbdTestCtl:
		test	kbdFlags, KBDCTRL
		jz	KbdTestCaps
		and	al, NOT 01100000b	; Clear out bits 6 & 5
KbdTestCaps:
		test	kbdFlags, KBDCAPS
		jz	KbdTestMeta
		cmp	al, 'a'
		jl	KbdTestMeta
		cmp	al, 'z'
		jg	KbdTestMeta
		sub	al, 'a' - 'A'		; Make upper-case
KbdTestMeta:
		test	kbdFlags, KBDMETA
		jz	KbdTestSys
		or	al, 80h
KbdTestSys:
		test	kbdFlags, KBDSYS
		jz	KbdIntStuff
		call	KbdHandleSys
		jc	KbdIntReturn		; Returns with carry clear if
						; AL should be stored
KbdIntStuff:
		call	KbdStuffChar		; Store modified key

KbdDiscard:
KbdIntReturn:
		mov	al, ICEOI
		out	ICEOIPORT, al		; Signal end-of-interrupt
		pop	bx
		pop	ax
		pop	ds
		iret

KbdIntRelease:
		tst	al
		jns	KbdDiscard		; We don't care about
						; non-special key releases
		cmp	al, MODLOW		; Or specials outside the
		jb	KbdDiscard		; modifier range
		cmp	al, MODHIGH
		ja	KbdDiscard
		sub	al, MODLOW

		;
		; Find bit for modifier in kbdKeyState
		;
		lea	bx, kbdModifiers
		xlatb

		;
		; Adjust kbdKeyState accordingly (clear the bit)
		;
		not	al
		and	al, kbdKeyState
		mov	kbdKeyState, al

		;
		; From the new state, find the new modifier bits for kbdFlags
		;
		mov	bx, offset kbdStateMap
		xlatb
		mov	ah, kbdFlags
		and	ah, NOT KBDMODFLAGS	; Clear out old values
		or	al, ah			; Set new values
		mov	kbdFlags, al
		jmp 	KbdIntReturn		; All done

KbdIntSpecial:
		;
		; A key was pressed that requires special handling
		;
		cmp	al, F1
		jb	KbdCheckMod
		call	KbdHandleFunc
		jmp	KbdIntReturn
KbdCheckMod:
		cmp	al, MODLOW
		jb	KbdCheckKeypad
		cmp	al, MODHIGH
		ja	KbdDoLock
		;
		; Modifier key pressed. Find appropriate state bits based on
		; the character and update kbdKeyState, then use just that
		; set of bits to index into the kbdStateMap to adjust kbdFlags
		; properly. Since we're just setting things, we don't need to
		; fetch the current state out of kbdFlags before looking things
		; up in kbdStateMap.
		;
		sub	al, MODLOW
		lea	bx, kbdModifiers
		xlatb			; Find modifier bit(s)
		or	kbdKeyState, al
		lea	bx, kbdStateMap
		xlatb
		or	kbdFlags, al	; Set appropriate bits
		jmp	KbdIntReturn
KbdCheckKeypad:
		test	kbdFlags, KBDKEYPADON
		jnz	KbdDoKeypad
		and	al, 7fh		; Keypad not on -- strip off special
					; bit and stuff the character
		cmp	al, '.'		; See if it's the Del key
		jne	KbdIntStuff	; Nope. Nothing special
		test	kbdFlags, KBDMETA OR KBDCTRL
		jz	KbdIntStuff	; If neither is on
		jnp	KbdIntStuff	; or only one is on, stuff a period
		;
		; We have a ctl-alt-del, so reset
		;
		mov	dx, KBDCMDPORT
		mov	al, KBDRESETCMD	; Strobe reset line
		out	dx, al		; Bye bye
KCK1:
		jmp	KbdDiscard	; Then again...
KbdDoKeypad:
		call	KbdHandleKeypad
		jmp	KbdIntReturn

KbdDoLock:
		;
		; A lock key was pressed. We only pay attention to the CapsLock
		; and NumLock keys for now.
		;
		cmp	al, CPL
		jne	KDL1
		mov	al, KBDCAPS
		jmp	short KDL2
KDL1:
		cmp	al, NML
		jne	KCK1		; Ignore it
		mov	al, KBDKEYPADON
KDL2:
		xor	al, kbdFlags
		or	al, KBDSETTINGLED
		mov	kbdFlags, al
		mov	al, KBDSETLEDCMD
		mov	lastByteSent, al
		out	KBDDATAPORT, al
		jmp	KbdIntReturn

		;
		; Handle receipt of an ACK byte from the keyboard. We should
		; only get such a byte if we're setting the LEDs, but we
		; test the flag just to make sure...
		;
KIGotAck:
		mov	ah, kbdFlags
		test	ah, KBDSETTINGLED
		jz	KBGA3
		clr	al
		test	ah, KBDCAPS
		jz	KBGA1
		or	al, 100b		; In CapsLock mode.
KBGA1:		test	ah, KBDKEYPADON
		jnz	KBGA2
		or	al, 010b		; Keypad producing numbers
KBGA2:		out	KBDDATAPORT, al

		mov	lastByteSent, al
		xor	ah, KBDSETTINGLED
		mov	kbdFlags, ah
KBGA3:
		jmp	KbdIntReturn
						; turn on NumLock LED
KbdInterrupt	endp
		bugsym	KbdInterrupt


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdHandleFunc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle the press of a function key

CALLED BY:	KbdInterrupt

PASS:		AL	= keycode of function key.

RETURN:		Nothing

DESTROYED:	AX, BX

PSEUDO CODE/STRATEGY:
	First see if the kbdFuncTrans table has a translation for the
	current combination of function and modifier keys and insert that
	translation into the keyboard buffer if so.

	Otherwise, form a default string based on the modifer/function combo
	and stuff that.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/13/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

KbdHandleFunc	proc	near
		push	cx

		and	al, 0fh		; Form function key #
		cbw			; Make it a word
		dec	ax		; Make 0-origin
		push	ax		; Save for later
		shl	ax, 1		; Shift by three to make room for
		shl	ax, 1		; modifier state
		shl	ax, 1
		clr	cx		; Zero-extend the flags
		mov	cl, kbdFlags
		and	cx, 7		; Get modifier bits
		or	ax, cx		; Form composite index
		shl	ax, 1		; Table is in words
		mov	bx, ax		; Must be in index reg
		mov	ax, kbdFuncTrans[bx]
		tst	ax
		jz	KbdFuncDefault	; No user-specified string -- make
					; default.
		push	si
		mov	si, ax
KbdFuncLoop:
		lodsb
		tst	al
		jz	KbdFuncLoopDone
		call	KbdStuffChar
		jmp	KbdFuncLoop
KbdFuncLoopDone:
		pop	si		; Restore si
		pop	ax		; Throw away function key #
		jmp	short KbdFuncReturn

KbdFuncDefault:
		mov	al, escape	; Stuff initial prefix (\E#)
		call	KbdStuffChar
		mov	al, '#'
		call	KbdStuffChar
		test	kbdFlags, KBDCTRL
		jz	KbdFuncAlt
		mov	al, 'c'		; Ctrl modifier -- stuff 'c'
		call	KbdStuffChar
KbdFuncAlt:
		test	kbdFlags, KBDMETA
		jz	KbdFuncShift
		mov	al, 'a'		; Alt modifier -- stuff 'a'
		call	KbdStuffChar
KbdFuncShift:
		test	kbdFlags, KBDSHIFT
		jz	KbdFuncNum
		mov	al, 's'		; Shift modifier -- stuff 's'
		call	KbdStuffChar
KbdFuncNum:
		pop	ax		; Fetch function number
		add	al, '0'		; Make it ascii
		call	KbdStuffChar	; Stuff it

KbdFuncReturn:
		pop	cx
		ret
KbdHandleFunc	endp
		bugsym	KbdHandleFunc


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdHandleKeypad
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle the pressing of a key on the keypad

CALLED BY:	KbdInterrupt

PASS:		AX	= The keycode of the pressed key

RETURN:		Nothing

DESTROYED:	AX, BX

PSEUDO CODE/STRATEGY:
	If shift key is down, revert to standard codes and stuff char.
	Else form \E# escape string and stuff it instead, using the data in
	kbdKeypadTrans to figure out the trailing character(s)

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/13/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

KbdHandleKeypad	proc	near
		;
		; Key is on keypad and keypad translation is on.
		; If shift key is also down, use regular ascii code. Otherwise,
		; index into kbdKeypadTrans to find one- or two-character
		; code to follow \E#
		;
		test	kbdFlags, KBDSHIFT
		jz	KbdTransKeypad
		and	al, 7fh
		call	KbdStuffChar
		jmp	KbdKPReturn
KbdTransKeypad:
		cmp	al, KPD		; See if it's the Del key (we have no
					; special translation for it, but want
					; to catch ctl-alt-del sequences)
		je	KbdHandleDel

		and	al, 0fh		; number is in low nibble
		shl	al, 1		; table is word-sized
		clr	ah
		push	ax		; Save index for later
		mov	al, escape
		call	KbdStuffChar
		mov	al, '#'
		call	KbdStuffChar
		pop	bx		; Restore index into bx
		mov	ax, kbdKeypadTrans[bx]
		call	KbdStuffChar
		mov	al, ah		; Get second character into al
		tst	al
		jz	KbdKPReturn	; zero => only one character
		call	KbdStuffChar
KbdKPReturn:
		ret
KbdHandleDel:
		;
		; See if both CTRL and ALT are down. If they are, PF will be
		; set (two bits on in result) and ZF will be off (non-zero
		; result).
		;
		test	kbdFlags, KBDMETA OR KBDCTRL
		jz	KbdNormalDel
		jnp	KbdNormalDel
		mov	dx, KBDCMDPORT
		mov	al, KBDRESETCMD
		out	dx, al
		jmp	KbdKPReturn
KbdNormalDel:
		;
		; Treat it like a period..
		;
		and	al, 7fh
		call	KbdStuffChar
		jmp	KbdKPReturn
KbdHandleKeypad	endp
		bugsym	KbdHandleKeypad


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdHandleSys
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle a keypress with the SysReq modifier

CALLED BY:	KbdInterrupt

PASS:		AL	= ascii code of character pressed

RETURN:		Carry Set if character should be discarded

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	The following commands are supported:
		Q	- quit: sets SF_EXIT bit sysFlags
		q	- quit: sets SF_EXIT bit sysFlags
		m	- mouse on: calls MouseStart
		M	- mouse off: calls MouseEnd
		f	- abort file transfer: set SF_NUKE_FILE bit of sysFlags
		s	- toggle screen-saver
		S	- go into swat on EC (SF_SWAT)
		^S	- go into swat (EC) passing /s (SF_SWAT)
		n	- go into swat on non-EC (SF_SWAT)
		^n	- go into swat (non-EC) passing /s (SF_SWAT)
		p	- print current screen (SF_PRINT_SCREEN)

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/13/88		Initial Revision
	adam	6/24/88		Added f command
	mike	1/27/88		added S command

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
KbdHandleSys	proc	near
		push	bx
		cmp	al, 'Q'
		jne	KHS1
		;
		; Q -- Quit command
		;
		or	sysFlags, MASK SF_EXIT
		jmp	short KHSRet
KHS1:		cmp	al, 'M'
		jne	KHS2
		;
		; M -- Mouse off
		;
		call	MouseEnd
		jmp	short KHSRet
KHS2:		cmp	al, 'm'
		jne	KHS3
		;
		; m -- Mouse on
		;
		call	MouseStart
		jmp	short KHSRet
KHS3:
		cmp	al, 'f'
		jne	KHS4
		;
		; f -- abort file transfer
		;
		or	sysFlags, MASK SF_NUKE_FILE
		jmp	short KHSRet
KHS4:
		;
		; s -- toggle screen saver state
		;
		cmp	al, 's'
		jne	KHS5
		call	ScrToggleSave
		jmp	short KHSRet
KHS5:
		;
		; c -- re-init com port
		;
		cmp	al, 'c'
		jne	KHS6
		call	ComExit
		call	ComInit
		jmp	short KHSRet
KHS6:
		;
		; S -- run swat with our arguments
		;
		cmp	al, 'S'
		jne	KHS7		; branch if not
		mov	bx, SA_RUN_EC
		jmp	KHSRunSwat
KHS7:
		;
		; ^S -- run swat /s + our arguments
		;
		cmp	al, 'S' AND 01fh
		jne	KHS8
		mov	bx, SA_STOP_EC
		jmp	KHSRunSwat
KHS8:
		;
		; n -- run swat on non-ec with our arguments
		;
		cmp	al, 'n'
		jne	KHS9		; branch if not
		mov	bx, SA_RUN_NEC
		jmp	KHSRunSwat
KHS9:
		;
		; ^n -- run swat (non-ec) /s + our arguments
		;
		cmp	al, 'n' AND 01fh
		jne	KHS10
		mov	bx, SA_STOP_NEC
KHSRunSwat:
		or	sysFlags, MASK SF_SWAT	; Tell Main to run swat
		add	bx, ds:[swatArgPtrs]
		mov	bx, ds:[bx]
		mov	ds:[swatArgs], bx
		jmp	short KHSRet
KHS10:
		;
		; q -- quit
		;
		cmp	al, 'q'
		jne	KHS11
		or	sysFlags, MASK SF_EXIT
		jmp	short KHSRet
KHS11:
		;
		; p -- print screen
		;
		cmp	al, 'p'
		jne	KHS12
		or	sysFlags, MASK SF_PRINT_SCREEN
		jmp	short KHSRet
KHS12:
KHSRet:
		pop	bx
		stc			; Discard all characters for now
		ret
KbdHandleSys	endp
		bugsym	KbdHandleSys



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdRead
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Read a character from the keyboard, if it's available

CALLED BY:	Main, KbdFunc

PASS:		Nothing

RETURN:		if ZF = 0, AL is character read
		else ZF = 1 and nothing returned.

DESTROYED:	AX, BX

PSEUDO CODE/STRATEGY:
	For now, call BIOS to fetch the character. If the character is 00,
	it's an extended sequence, so we call KbdFunc to handle it.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	KbdRead
KbdRead		proc	near
		dsi
		mov	bx, kbdBuf.kb_head
		cmp	bx, kbdBuf.kb_tail
		eni

		je	KbdReadRet

		mov	al, kbdBuf.kb_data[bx]

		inc	bx
		cmp	bx, KBBUFSIZE
		jne	KbdReadStoreHead
		clr	bx
KbdReadStoreHead:
		mov	kbdBuf.kb_head, bx
		or	bx, 1		; Return non-zero
KbdReadRet:
		ret
KbdRead		endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdKeypadStart
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Turn on translation of keypad keys to escape codes

CALLED BY:	ScrParseSeq (INTERNAL to Scr module)

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AL

PSEUDO CODE/STRATEGY:
	None

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision
	adam	7/21/88		Changed to set LEDs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	KbdKeypadStart
KbdKeypadStart	proc	near
		or	kbdFlags, KBDKEYPADON OR KBDSETTINGLED
		mov	al, KBDSETLEDCMD
		out	KBDDATAPORT, al
		ret
KbdKeypadStart	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdKeypadEnd
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Change back to transmitting numbers for keypad

CALLED BY:	ScrParseSeq (INTERNAL to Scr module)

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AL

PSEUDO CODE/STRATEGY:
	None, yet

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision
	adam	7/21/88		Changed to set LEDs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	KbdKeypadEnd
KbdKeypadEnd	proc	near
		and	kbdFlags, NOT KBDKEYPADON
		or	kbdFlags, KBDSETTINGLED
		mov	al, KBDSETLEDCMD
		out	KBDDATAPORT, al
		ret
KbdKeypadEnd	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdGetMods
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Return a bitmask of the current modifier keys

CALLED BY:	MouseEvent

PASS:		Nothing

RETURN:		AL	= mask:
			bit 0 = shift
			bit 1 = meta
			bit 2 = ctrl
			bit 4 = sys

DESTROYED:	AL

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	KbdGetMods
KbdGetMods	proc	near
		mov	al, kbdFlags
		and	al, 1111b
		ret
KbdGetMods	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		KbdInsertString
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Insert the given string as keyboard input

CALLED BY:	MouseEvent

PASS:		DS:SI	= string to store
		CX	= number of characters in string

RETURN:		Nothing

DESTROYED:	CX, AX, SI

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	KbdInsertString
KbdInsertString	proc	near
		dsi			; Add chars atomically
		push	bx
		jcxz	KISRet
		mov	bx, kbdBuf.kb_tail
		cld			; Move up through the string.
KIS1:		lodsb			; Fetch next char
		mov	kbdBuf.kb_data[bx], al	; Stuff it
		inc	bx		; Update tail pointer and wrap if nec'y
		cmp	bx, KBBUFSIZE
		jne	KIS2
		clr	bx
KIS2:
		cmp	bx, kbdBuf.kb_head	; Overflow?
		je	KISRet		; String too long to fit in buffer,
					; so discard the whole thing
		loop	KIS1
		mov	kbdBuf.kb_tail, bx
KISRet:
		pop	bx
		eni
		ret
KbdInsertString	endp

program		ends

		end
